/**
 * Copyright (c) 2007, 2010, 2013, 2017 Borland Software Corporation and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 * Dmitry Stadnik (Borland) - initial API and implementation
 * Michael Golubev (Montages) - #386838 - migrate to Xtend2
 * Vincent Lorenzo (CEA-LIST) - Bug 520882
 */
package aspects.xpt.providers;

import aspects.xpt.Common;
import aspects.xpt.providers.ViewProvider;
import aspects.xpt.providers.VisualTypeProvider;
import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.util.Arrays;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gmf.codegen.gmfgen.ElementType;
import org.eclipse.gmf.codegen.gmfgen.GenCommonBase;
import org.eclipse.gmf.codegen.gmfgen.GenDiagram;
import org.eclipse.gmf.codegen.gmfgen.GenNode;
import org.eclipse.gmf.codegen.gmfgen.GenNodeLabel;
import org.eclipse.gmf.codegen.gmfgen.MetamodelType;
import org.eclipse.gmf.codegen.gmfgen.NotationType;
import org.eclipse.gmf.codegen.gmfgen.ProviderPriority;
import org.eclipse.gmf.codegen.gmfgen.SpecializationType;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import parsers.ParserProvider;
import xpt.diagram.Utils_qvto;
import xpt.diagram.edithelpers.EditHelper;
import xpt.diagram.edithelpers.EditHelperAdvice;
import xpt.providers.EditPartProvider;
import xpt.providers.IconProvider;
import xpt.providers.ShortcutsDecoratorProvider;

@Singleton
@SuppressWarnings("all")
public class extensions extends xpt.providers.extensions {
  @Inject
  @Extension
  private Common _common;
  
  @Inject
  @Extension
  private Utils_qvto _utils_qvto;
  
  @Inject
  private ViewProvider viewProvider;
  
  @Inject
  private IconProvider iconProvider;
  
  @Inject
  private EditPartProvider editPartProvider;
  
  @Inject
  private ParserProvider labelParsers;
  
  @Inject
  private ShortcutsDecoratorProvider shorcutProvider;
  
  @Inject
  private VisualTypeProvider visualTypeProvider;
  
  @Inject
  private EditHelper editHelper;
  
  @Inject
  private EditHelperAdvice editHelperAdvice;
  
  @Override
  public CharSequence extensions(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("\t");
    CharSequence _extraLineBreak = this._common.extraLineBreak();
    _builder.append(_extraLineBreak, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace = this._common.tripleSpace(1);
    _builder.append(_tripleSpace, "\t");
    _builder.append("<extension point=\"org.eclipse.gmf.runtime.diagram.core.viewProviders\" id=\"view-provider\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_1 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_1, "\t");
    CharSequence _xmlGeneratedTag = this._common.xmlGeneratedTag();
    _builder.append(_xmlGeneratedTag, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_2 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_2, "\t");
    _builder.append("<viewProvider class=\"");
    CharSequence _qualifiedClassName = this.viewProvider.qualifiedClassName(it);
    _builder.append(_qualifiedClassName, "\t");
    _builder.append("\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_3 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_3, "\t");
    _builder.append("<Priority name=\"");
    ProviderPriority _notationViewProviderPriority = it.getNotationViewProviderPriority();
    _builder.append(_notationViewProviderPriority, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    {
      boolean _isEmpty = it.getShortcutsProvidedFor().isEmpty();
      boolean _not = (!_isEmpty);
      if (_not) {
        String _tripleSpace_4 = this._common.tripleSpace(3);
        _builder.append(_tripleSpace_4);
        _builder.append("<object id=\"referencing-diagrams\" class=\"org.eclipse.gmf.runtime.notation.Diagram\">");
        _builder.newLineIfNotEmpty();
        String _tripleSpace_5 = this._common.tripleSpace(4);
        _builder.append(_tripleSpace_5);
        _builder.append("<method name=\"getType()\" value=\"");
        {
          EList<String> _shortcutsProvidedFor = it.getShortcutsProvidedFor();
          boolean _hasElements = false;
          for(final String s : _shortcutsProvidedFor) {
            if (!_hasElements) {
              _hasElements = true;
            } else {
              _builder.appendImmediate(",", "");
            }
            _builder.append(s);
          }
        }
        _builder.append("\"/>");
        _builder.newLineIfNotEmpty();
        String _tripleSpace_6 = this._common.tripleSpace(3);
        _builder.append(_tripleSpace_6);
        _builder.append("</object>");
        _builder.newLineIfNotEmpty();
        String _tripleSpace_7 = this._common.tripleSpace(3);
        _builder.append(_tripleSpace_7);
        _builder.append("<context viewClass=\"org.eclipse.gmf.runtime.notation.Node\" containerViews=\"referencing-diagrams\"/>");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    String _tripleSpace_8 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_8, "\t");
    _builder.append("<context viewClass=\"org.eclipse.gmf.runtime.notation.Diagram\" semanticHints=\"");
    String _modelID = it.getEditorGen().getModelID();
    _builder.append(_modelID, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_9 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_9, "\t");
    _builder.append("<context viewClass=\"org.eclipse.gmf.runtime.notation.Node\" semanticHints=\"");
    CharSequence _commaSeparatedVisualIDs = this.commaSeparatedVisualIDs(it.getAllNodes());
    _builder.append(_commaSeparatedVisualIDs, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    {
      boolean _isEmpty_1 = it.getLinks().isEmpty();
      boolean _not_1 = (!_isEmpty_1);
      if (_not_1) {
        String _tripleSpace_10 = this._common.tripleSpace(3);
        _builder.append(_tripleSpace_10);
        _builder.append("<context viewClass=\"org.eclipse.gmf.runtime.notation.Edge\" semanticHints=\"");
        CharSequence _commaSeparatedVisualIDs_1 = this.commaSeparatedVisualIDs(it.getLinks());
        _builder.append(_commaSeparatedVisualIDs_1);
        _builder.append("\"/>");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    String _tripleSpace_11 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_11, "\t");
    _builder.append("</viewProvider>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_12 = this._common.tripleSpace(1);
    _builder.append(_tripleSpace_12, "\t");
    _builder.append("</extension>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    String _tripleSpace_13 = this._common.tripleSpace(1);
    _builder.append(_tripleSpace_13, "\t");
    _builder.append("<extension point=\"org.eclipse.gmf.runtime.diagram.ui.editpartProviders\" id=\"ep-provider\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_14 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_14, "\t");
    CharSequence _xmlGeneratedTag_1 = this._common.xmlGeneratedTag();
    _builder.append(_xmlGeneratedTag_1, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_15 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_15, "\t");
    _builder.append("<editpartProvider class=\"");
    CharSequence _qualifiedClassName_1 = this.editPartProvider.qualifiedClassName(it);
    _builder.append(_qualifiedClassName_1, "\t");
    _builder.append("\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_16 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_16, "\t");
    _builder.append("<Priority name=\"");
    ProviderPriority _editPartProviderPriority = it.getEditPartProviderPriority();
    _builder.append(_editPartProviderPriority, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_17 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_17, "\t");
    _builder.append("<object class=\"org.eclipse.gmf.runtime.notation.Diagram\" id=\"generated-diagram\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_18 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_18, "\t");
    _builder.append("<method name=\"getType()\" value=\"");
    String _modelID_1 = it.getEditorGen().getModelID();
    _builder.append(_modelID_1, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_19 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_19, "\t");
    _builder.append("</object>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_20 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_20, "\t");
    _builder.append("<object class=\"org.eclipse.gmf.runtime.notation.Node\" id=\"generated-nodes\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_21 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_21, "\t");
    _builder.append("<method name=\"getType()\" value=\"");
    CharSequence _commaSeparatedVisualIDs_2 = this.commaSeparatedVisualIDs(it.getAllNodes());
    _builder.append(_commaSeparatedVisualIDs_2, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_22 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_22, "\t");
    _builder.append("</object>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_23 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_23, "\t");
    _builder.append("<object class=\"org.eclipse.gmf.runtime.notation.Edge\" id=\"generated-links\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_24 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_24, "\t");
    _builder.append("<method name=\"getType()\" value=\"");
    CharSequence _commaSeparatedVisualIDs_3 = this.commaSeparatedVisualIDs(it.getLinks());
    _builder.append(_commaSeparatedVisualIDs_3, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_25 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_25, "\t");
    _builder.append("</object>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_26 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_26, "\t");
    _builder.append("<object class=\"org.eclipse.gmf.runtime.notation.Node\" id=\"generated-labels\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_27 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_27, "\t");
    _builder.append("<method name=\"getType()\" value=\"");
    final Function1<GenNode, EList<GenNodeLabel>> _function = new Function1<GenNode, EList<GenNodeLabel>>() {
      @Override
      public EList<GenNodeLabel> apply(final GenNode n) {
        return n.getLabels();
      }
    };
    CharSequence _commaSeparatedVisualIDs_4 = this.commaSeparatedVisualIDs(Iterables.<GenNodeLabel>concat(ListExtensions.<GenNode, EList<GenNodeLabel>>map(it.getAllNodes(), _function)));
    _builder.append(_commaSeparatedVisualIDs_4, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_28 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_28, "\t");
    _builder.append("</object>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_29 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_29, "\t");
    _builder.append("<object class=\"org.eclipse.gmf.runtime.notation.Node\" id=\"generated-compartments\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_30 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_30, "\t");
    _builder.append("<method name=\"getType()\" value=\"");
    CharSequence _commaSeparatedVisualIDs_5 = this.commaSeparatedVisualIDs(it.getCompartments());
    _builder.append(_commaSeparatedVisualIDs_5, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_31 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_31, "\t");
    _builder.append("</object>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_32 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_32, "\t");
    _builder.append("<context views=\"generated-diagram,generated-nodes,generated-links,generated-labels,generated-compartments\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_33 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_33, "\t");
    _builder.append("</editpartProvider>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_34 = this._common.tripleSpace(1);
    _builder.append(_tripleSpace_34, "\t");
    _builder.append("</extension>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("\t");
    String _tripleSpace_35 = this._common.tripleSpace(1);
    _builder.append(_tripleSpace_35, "\t");
    _builder.append("<extension point=\"org.eclipse.gmf.runtime.common.ui.services.iconProviders\" id=\"icon-provider\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_36 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_36, "\t");
    CharSequence _xmlGeneratedTag_2 = this._common.xmlGeneratedTag();
    _builder.append(_xmlGeneratedTag_2, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_37 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_37, "\t");
    _builder.append("<IconProvider class=\"");
    CharSequence _qualifiedClassName_2 = this.iconProvider.qualifiedClassName(it);
    _builder.append(_qualifiedClassName_2, "\t");
    _builder.append("\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_38 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_38, "\t");
    _builder.append("<Priority name=\"");
    ProviderPriority _iconProviderPriority = it.getIconProviderPriority();
    _builder.append(_iconProviderPriority, "\t");
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_39 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_39, "\t");
    _builder.append("</IconProvider>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_40 = this._common.tripleSpace(1);
    _builder.append(_tripleSpace_40, "\t");
    _builder.append("</extension>");
    _builder.newLineIfNotEmpty();
    {
      if (((!Objects.equal(it.getEditorGen().getLabelParsers(), null)) && it.getEditorGen().getLabelParsers().isExtensibleViaService())) {
        _builder.append("\t");
        CharSequence _extraLineBreak_1 = this._common.extraLineBreak();
        _builder.append(_extraLineBreak_1, "\t");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_41 = this._common.tripleSpace(1);
        _builder.append(_tripleSpace_41, "\t");
        _builder.append("<extension point=\"org.eclipse.gmf.runtime.common.ui.services.parserProviders\" id=\"parser-provider\">");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_42 = this._common.tripleSpace(2);
        _builder.append(_tripleSpace_42, "\t");
        CharSequence _xmlGeneratedTag_3 = this._common.xmlGeneratedTag();
        _builder.append(_xmlGeneratedTag_3, "\t");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_43 = this._common.tripleSpace(2);
        _builder.append(_tripleSpace_43, "\t");
        _builder.append("<ParserProvider class=\"");
        CharSequence _qualifiedClassName_3 = this.labelParsers.qualifiedClassName(it.getEditorGen().getLabelParsers());
        _builder.append(_qualifiedClassName_3, "\t");
        _builder.append("\">");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_44 = this._common.tripleSpace(3);
        _builder.append(_tripleSpace_44, "\t");
        _builder.append("<Priority name=\"");
        ProviderPriority _providerPriority = it.getEditorGen().getLabelParsers().getProviderPriority();
        _builder.append(_providerPriority, "\t");
        _builder.append("\"/>");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_45 = this._common.tripleSpace(2);
        _builder.append(_tripleSpace_45, "\t");
        _builder.append("</ParserProvider>");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_46 = this._common.tripleSpace(1);
        _builder.append(_tripleSpace_46, "\t");
        _builder.append("</extension>");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      boolean _generateShortcutIcon = it.generateShortcutIcon();
      if (_generateShortcutIcon) {
        _builder.append("\t");
        CharSequence _extraLineBreak_2 = this._common.extraLineBreak();
        _builder.append(_extraLineBreak_2, "\t");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_47 = this._common.tripleSpace(1);
        _builder.append(_tripleSpace_47, "\t");
        _builder.append("<extension point=\"org.eclipse.gmf.runtime.diagram.ui.decoratorProviders\" id=\"decorator-provider\">");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_48 = this._common.tripleSpace(2);
        _builder.append(_tripleSpace_48, "\t");
        CharSequence _xmlGeneratedTag_4 = this._common.xmlGeneratedTag();
        _builder.append(_xmlGeneratedTag_4, "\t");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_49 = this._common.tripleSpace(2);
        _builder.append(_tripleSpace_49, "\t");
        _builder.append("<decoratorProvider class=\"");
        CharSequence _qualifiedClassName_4 = this.shorcutProvider.qualifiedClassName(it);
        _builder.append(_qualifiedClassName_4, "\t");
        _builder.append("\">");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_50 = this._common.tripleSpace(3);
        _builder.append(_tripleSpace_50, "\t");
        _builder.append("<Priority name=\"");
        ProviderPriority _shortcutsDecoratorProviderPriority = it.getShortcutsDecoratorProviderPriority();
        _builder.append(_shortcutsDecoratorProviderPriority, "\t");
        _builder.append("\"/>");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_51 = this._common.tripleSpace(3);
        _builder.append(_tripleSpace_51, "\t");
        _builder.append("<object class=\"org.eclipse.gmf.runtime.notation.Node(org.eclipse.gmf.runtime.notation)\" id=\"generated-top-nodes\">");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_52 = this._common.tripleSpace(4);
        _builder.append(_tripleSpace_52, "\t");
        _builder.append("<method name=\"getType()\" value=\"");
        CharSequence _commaSeparatedVisualIDs_6 = this.commaSeparatedVisualIDs(it.getTopLevelNodes());
        _builder.append(_commaSeparatedVisualIDs_6, "\t");
        _builder.append("\"/>");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_53 = this._common.tripleSpace(3);
        _builder.append(_tripleSpace_53, "\t");
        _builder.append("</object>");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_54 = this._common.tripleSpace(3);
        _builder.append(_tripleSpace_54, "\t");
        _builder.append("<context decoratorTargets=\"generated-top-nodes\"/>");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_55 = this._common.tripleSpace(2);
        _builder.append(_tripleSpace_55, "\t");
        _builder.append("</decoratorProvider>");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        String _tripleSpace_56 = this._common.tripleSpace(1);
        _builder.append(_tripleSpace_56, "\t");
        _builder.append("</extension>");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _extraLineBreak_3 = this._common.extraLineBreak();
    _builder.append(_extraLineBreak_3, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_57 = this._common.tripleSpace(1);
    _builder.append(_tripleSpace_57, "\t");
    _builder.append("<extension point=\"org.eclipse.papyrus.infra.gmfdiag.common.visualTypeProviders\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_58 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_58, "\t");
    CharSequence _xmlGeneratedTag_5 = this._common.xmlGeneratedTag();
    _builder.append(_xmlGeneratedTag_5, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_59 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_59, "\t");
    _builder.append("<visualTypeProvider");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_60 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_60, "\t");
    _builder.append("class=\"");
    CharSequence _qualifiedClassName_5 = this.visualTypeProvider.qualifiedClassName(it);
    _builder.append(_qualifiedClassName_5, "\t");
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_61 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_61, "\t");
    _builder.append("diagramType=\"");
    String _modelID_2 = it.getEditorGen().getModelID();
    _builder.append(_modelID_2, "\t");
    _builder.append("\">");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_62 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_62, "\t");
    _builder.append("</visualTypeProvider>");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    String _tripleSpace_63 = this._common.tripleSpace(1);
    _builder.append(_tripleSpace_63, "\t");
    _builder.append("</extension>");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public Iterable<GenCommonBase> getLocalDefineTypedElements(final GenDiagram it) {
    final Function1<GenCommonBase, Boolean> _function = new Function1<GenCommonBase, Boolean>() {
      @Override
      public Boolean apply(final GenCommonBase et) {
        boolean _isDefinedExternally = et.getElementType().isDefinedExternally();
        return Boolean.valueOf((false == _isDefinedExternally));
      }
    };
    return IterableExtensions.<GenCommonBase>filter(this._utils_qvto.getAllTypedElements(it), _function);
  }
  
  @Override
  public CharSequence commaSeparatedVisualIDs(final Iterable<? extends GenCommonBase> list) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _hasElements = false;
      for(final GenCommonBase gcb : list) {
        if (!_hasElements) {
          _hasElements = true;
        } else {
          _builder.appendImmediate(",", "");
        }
        String _stringUniqueIdentifier = this._common.stringUniqueIdentifier(gcb);
        _builder.append(_stringUniqueIdentifier);
      }
    }
    return _builder;
  }
  
  @Override
  protected CharSequence _elementType(final MetamodelType it) {
    StringConcatenation _builder = new StringConcatenation();
    String _tripleSpace = this._common.tripleSpace(2);
    _builder.append(_tripleSpace);
    _builder.append("<metamodel nsURI=\"");
    String _nsURI = it.getMetaClass().getGenPackage().getEcorePackage().getNsURI();
    _builder.append(_nsURI);
    _builder.append("\">");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_1 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_1);
    _builder.append("<metamodelType");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_2 = this._common.tripleSpace(5);
    _builder.append(_tripleSpace_2);
    _builder.append("id=\"");
    String _uniqueIdentifier = it.getUniqueIdentifier();
    _builder.append(_uniqueIdentifier);
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    {
      String _displayName = it.getDisplayName();
      boolean _notEquals = (!Objects.equal(null, _displayName));
      if (_notEquals) {
        String _tripleSpace_3 = this._common.tripleSpace(5);
        _builder.append(_tripleSpace_3);
        _builder.append("name=\"%metatype.name.");
        String _stringUniqueIdentifier = this._common.stringUniqueIdentifier(it.getDiagramElement());
        _builder.append(_stringUniqueIdentifier);
        _builder.append("\"");
        _builder.newLineIfNotEmpty();
      }
    }
    String _tripleSpace_4 = this._common.tripleSpace(5);
    _builder.append(_tripleSpace_4);
    _builder.append("kind=\"org.eclipse.gmf.runtime.emf.type.core.IHintedType\"");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_5 = this._common.tripleSpace(5);
    _builder.append(_tripleSpace_5);
    _builder.append("eclass=\"");
    String _name = it.getMetaClass().getEcoreClass().getName();
    _builder.append(_name);
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_6 = this._common.tripleSpace(5);
    _builder.append(_tripleSpace_6);
    _builder.append("edithelper=\"");
    CharSequence _qualifiedClassName = this.editHelper.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    _builder.append("\">");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_7 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_7);
    _builder.append("<param name=\"semanticHint\" value=\"");
    String _stringVisualID = this._common.stringVisualID(it.getDiagramElement());
    _builder.append(_stringVisualID);
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_8 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_8);
    _builder.append("</metamodelType>");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_9 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_9);
    _builder.append("</metamodel>");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  @Override
  public CharSequence specializationType(final SpecializationType it) {
    StringConcatenation _builder = new StringConcatenation();
    String _tripleSpace = this._common.tripleSpace(3);
    _builder.append(_tripleSpace);
    _builder.append("<specializationType");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_1 = this._common.tripleSpace(5);
    _builder.append(_tripleSpace_1);
    _builder.append("id=\"");
    String _uniqueIdentifier = it.getUniqueIdentifier();
    _builder.append(_uniqueIdentifier);
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    {
      String _displayName = it.getDisplayName();
      boolean _notEquals = (!Objects.equal(null, _displayName));
      if (_notEquals) {
        String _tripleSpace_2 = this._common.tripleSpace(5);
        _builder.append(_tripleSpace_2);
        _builder.append("name=\"%metatype.name.");
        String _stringUniqueIdentifier = this._common.stringUniqueIdentifier(it.getDiagramElement());
        _builder.append(_stringUniqueIdentifier);
        _builder.append("\"");
        _builder.newLineIfNotEmpty();
      }
    }
    String _tripleSpace_3 = this._common.tripleSpace(5);
    _builder.append(_tripleSpace_3);
    _builder.append("kind=\"org.eclipse.gmf.runtime.emf.type.core.IHintedType\"");
    {
      String _editHelperAdviceClassName = it.getEditHelperAdviceClassName();
      boolean _notEquals_1 = (!Objects.equal(_editHelperAdviceClassName, null));
      if (_notEquals_1) {
        _builder.newLineIfNotEmpty();
        String _tripleSpace_4 = this._common.tripleSpace(5);
        _builder.append(_tripleSpace_4);
        _builder.append("edithelperadvice=\"");
        CharSequence _qualifiedClassName = this.editHelperAdvice.qualifiedClassName(it);
        _builder.append(_qualifiedClassName);
        _builder.append("\"");
      }
    }
    _builder.append(">");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_5 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_5);
    _builder.append("<specializes id=\"");
    {
      MetamodelType _metamodelType = it.getMetamodelType();
      boolean _equals = Objects.equal(null, _metamodelType);
      if (_equals) {
        _builder.append("org.eclipse.gmf.runtime.emf.type.core.null");
      } else {
        String _uniqueIdentifier_1 = it.getMetamodelType().getUniqueIdentifier();
        _builder.append(_uniqueIdentifier_1);
      }
    }
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_6 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_6);
    _builder.append("<param name=\"semanticHint\" value=\"");
    String _stringVisualID = this._common.stringVisualID(it.getDiagramElement());
    _builder.append(_stringVisualID);
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_7 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_7);
    _builder.append("</specializationType>");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  @Override
  protected CharSequence _elementType(final NotationType it) {
    StringConcatenation _builder = new StringConcatenation();
    String _tripleSpace = this._common.tripleSpace(2);
    _builder.append(_tripleSpace);
    _builder.append("<specializationType");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_1 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_1);
    _builder.append("id=\"");
    String _uniqueIdentifier = it.getUniqueIdentifier();
    _builder.append(_uniqueIdentifier);
    _builder.append("\"");
    _builder.newLineIfNotEmpty();
    {
      String _displayName = it.getDisplayName();
      boolean _notEquals = (!Objects.equal(null, _displayName));
      if (_notEquals) {
        String _tripleSpace_2 = this._common.tripleSpace(4);
        _builder.append(_tripleSpace_2);
        _builder.append("name=\"%metatype.name.");
        String _stringUniqueIdentifier = this._common.stringUniqueIdentifier(it.getDiagramElement());
        _builder.append(_stringUniqueIdentifier);
        _builder.append("\"");
        _builder.newLineIfNotEmpty();
      }
    }
    String _tripleSpace_3 = this._common.tripleSpace(4);
    _builder.append(_tripleSpace_3);
    _builder.append("kind=\"org.eclipse.gmf.runtime.diagram.ui.util.INotationType\">");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_4 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_4);
    _builder.append("<specializes id=\"org.eclipse.gmf.runtime.emf.type.core.null\"/>");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_5 = this._common.tripleSpace(3);
    _builder.append(_tripleSpace_5);
    _builder.append("<param name=\"semanticHint\" value=\"");
    String _stringVisualID = this._common.stringVisualID(it.getDiagramElement());
    _builder.append(_stringVisualID);
    _builder.append("\"/>");
    _builder.newLineIfNotEmpty();
    String _tripleSpace_6 = this._common.tripleSpace(2);
    _builder.append(_tripleSpace_6);
    _builder.append("</specializationType>");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  @Override
  public CharSequence elementType(final ElementType it) {
    if (it instanceof MetamodelType) {
      return _elementType((MetamodelType)it);
    } else if (it instanceof NotationType) {
      return _elementType((NotationType)it);
    } else if (it instanceof SpecializationType) {
      return _elementType((SpecializationType)it);
    } else if (it != null) {
      return _elementType(it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
}
